package com.dragoninfo.dcuc.app.facade;

import com.dragoninfo.dcuc.app.entity.ManufacturerInfo;
import com.dragonsoft.duceap.base.entity.metadata.CodeRecord;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * @author yica
 * @date 2018/8/8
 */
@FeignClient(name = "dcuc-app", path = "/dcuc/app/manufacturerInfoFacade")
public interface IManufacturerInfoFacade {
    /**
     * 分页查询
     *
     * @param searchDTO 搜索条件
     * @return 厂商列表
     */
    @PostMapping(value = "manufacturerInfoPage")
    Page<ManufacturerInfo> manufacturerInfoPage(@RequestBody SearchDTO searchDTO);

    /**
     * 详情
     *
     * @param id 厂商ID
     * @return 厂商信息
     */
    @RequestMapping(value = "manufacturerInfoDetail")
    ManufacturerInfo manufacturerInfoDetail(@RequestParam("id") String id);


    /**
     * 通过厂商代码获取厂商
     *
     * @param code 厂商代码
     * @return 厂商信息
     */
    @RequestMapping(value = "detailByCode")
    ManufacturerInfo detailByCode(@RequestParam("code") String code);

    /**
     * 根据名称获取厂商
     *
     * @param manufacturerName 厂商名称
     * @return 厂商详情
     */
    @GetMapping(value = "manufacturerName")
    ManufacturerInfo getOneByName(@RequestParam("manufacturerName") String manufacturerName);

    /**
     * 根据代码、名称获取厂商
     *
     * @param code             厂商代码
     * @param manufacturerName 厂商名称
     * @return 厂商详情
     */
    @RequestMapping(value = "getOneByCodeName")
    ManufacturerInfo getOneByCodeName(@RequestParam("code") String code, @RequestParam("manufacturerName") String manufacturerName);

    /**
     * 获取所有厂商
     *
     * @return List<CodeRecord>
     */
    @RequestMapping(value = "getAllCodeRecord")
    List<CodeRecord> getAllCodeRecord();

    /**
     * 获取所有 Key{id},Value{name} 的 Map
     *
     * @return Key{id},Value{name} 的 Map
     */
    @RequestMapping(value = "getAllIdNameMap")
    Map<String, String> getAllIdNameMap();

    /**
     * 获取所有厂商
     *
     * @return 所有厂商
     */
    @RequestMapping(value = "getAllList")
    List<ManufacturerInfo> getAllList();

    /**
     * 保存
     *
     * @param manufacturerInfo 厂商信息
     * @return 厂商信息
     */
    @PostMapping(value = "saveManufacturer")
    ManufacturerInfo saveManufacturer(@RequestBody ManufacturerInfo manufacturerInfo);

    /**
     * 修改
     *
     * @param manufacturerInfo 厂商信息
     * @return 厂商信息
     */
    @PostMapping(value = "updateManufacturer")
    ManufacturerInfo updateManufacturer(@RequestBody ManufacturerInfo manufacturerInfo);

    /**
     * 删除
     *
     * @param id id
     */
    @RequestMapping(value = "del")
    void del(@RequestParam("id") String id);

}
