package com.dragoninfo.dcuc.app.business.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.util.PageUtil;
import com.alibaba.fastjson.JSON;
import com.dragoninfo.dcuc.app.business.IDataCatalogBusiness;
import com.dragoninfo.dcuc.app.config.DcucResourceConfig;
import com.dragoninfo.dcuc.app.dto.dataresource.*;
import com.dragoninfo.dcuc.app.entity.sub.AppDataItemInfo;
import com.dragoninfo.dcuc.app.entity.sub.AppDataResourceInfo;
import com.dragoninfo.dcuc.app.service.sub.IAppDataItemInfoService;
import com.dragoninfo.dcuc.app.service.sub.IAppDataResourceInfoService;
import com.dragoninfo.dcuc.common.http.HttpUtil;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.commons.util.UUIDUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.google.common.reflect.TypeToken;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.lang.reflect.Type;
import java.net.URI;
import java.util.*;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2022/10/2
 */
@Slf4j
@Component
public class DataCatalogBusinessImpl implements IDataCatalogBusiness {

    @Autowired
    private DcucResourceConfig dataResourceConfig;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private IAppDataResourceInfoService appDataResourceInfoService;

    @Autowired
    private IAppDataItemInfoService appDataItemInfoService;
    /**
     * 数据类别查询接口
     */
    public static final String RESOURCE_TYPE = "/resource/catalog-api/resource-type";

    /**
     * 资源列表查询
     */
    public static final String CATALOG_LIST_URL = "/resource/catalog-api/catalog-list";

    @Async
    @Override
    public void syncTableAndColumn() {
        // 获取总条数
        int totalCount = getTotalCount();
        Integer requestPageSize = dataResourceConfig.getDataResource().getRequestPageSize();
        int totalPage = PageUtil.totalPage(totalCount, requestPageSize);
        int reqPage = 1;
        CopyOptions ignoreNullCopyOptions = CopyOptions.create().setIgnoreNullValue(true);

        do {
            PageReqDTO pageReqDTO = new PageReqDTO();
            pageReqDTO.setPage(reqPage);
            pageReqDTO.setPageSize(requestPageSize);

            Optional<PageDataRespDTO<DataResourceInfoDTO>> pageOptional = getPage(pageReqDTO);
            if (pageOptional.isPresent()) {
                PageDataRespDTO<DataResourceInfoDTO> pageDataRespDTO = pageOptional.get();
                List<DataResourceInfoDTO> dataList = pageDataRespDTO.getDataList();

                List<AppDataResourceInfo> addResourceInfoList = new ArrayList<>();
                List<AppDataResourceInfo> updateResourceInfoList = new ArrayList<>();

                List<AppDataItemInfo> addDataItemInfoList = new ArrayList<>();
                List<AppDataItemInfo> updateDataItemInfoList = new ArrayList<>();

                for (DataResourceInfoDTO dataResourceInfoDTO : dataList) {
                    String resourceCode = dataResourceInfoDTO.getResourceCode();

                    AppDataResourceInfo appDataResourceInfo = appDataResourceInfoService.findByResourceCode(resourceCode);
                    if (appDataResourceInfo == null) {
                        appDataResourceInfo = dataResourceInfoDTO.parseToAppDataResourceInfo();
                        String uuid = UUIDUtils.getUUID();
                        appDataResourceInfo.setId(uuid);
                        appDataResourceInfo.setDeleted(BooleanEnum.FALSE.value);
                        addResourceInfoList.add(appDataResourceInfo);
                    } else {
                        AppDataResourceInfo newAppDataResourceInfo = dataResourceInfoDTO.parseToAppDataResourceInfo();
                        BeanUtil.copyProperties(newAppDataResourceInfo, appDataResourceInfo, ignoreNullCopyOptions);
                        updateResourceInfoList.add(appDataResourceInfo);
                    }


                    List<DataResourceItemIntoDTO> dataItems = dataResourceInfoDTO.getDataItems();
                    for (DataResourceItemIntoDTO dataItem : dataItems) {
                        String id = appDataResourceInfo.getId();
                        String dataItemNo = dataItem.getDataItemNo();
                        AppDataItemInfo appDataItemInfo = appDataItemInfoService.selectByDataItemNo(dataItemNo);
                        if (appDataItemInfo == null) {
                            appDataItemInfo = dataItem.parseToAppDataItemInfo();
                            appDataItemInfo.setDeleted(BooleanEnum.FALSE.value);
                            appDataItemInfo.setResourceTableId(id);
                            addDataItemInfoList.add(appDataItemInfo);
                        } else {
                            AppDataItemInfo newAppDataItemInfo = dataItem.parseToAppDataItemInfo();
                            BeanUtil.copyProperties(newAppDataItemInfo, appDataItemInfo, ignoreNullCopyOptions);
                            updateDataItemInfoList.add(appDataItemInfo);
                        }

                    }
                }

                appDataResourceInfoService.saveAll(addResourceInfoList);
                appDataResourceInfoService.updateAll(updateResourceInfoList);
                appDataItemInfoService.saveAll(addDataItemInfoList);
                appDataItemInfoService.updateAll(updateDataItemInfoList);
            }

            reqPage++;
        } while (reqPage <= totalPage);
    }

    /**
     * 获取总条数
     *
     * @return 总条数
     */
    public int getTotalCount() {
        PageReqDTO pageReqDTO = new PageReqDTO();
        pageReqDTO.setPage(1);
        pageReqDTO.setPageSize(1);
        Optional<PageDataRespDTO<DataResourceInfoDTO>> pageOptional = getPage(pageReqDTO);
        int totalCount = 0;
        if (pageOptional.isPresent()) {
            totalCount = pageOptional.get().getTotalCount();
        }
        return totalCount;
    }

    /**
     * 获取分页信息
     *
     * @param pageReqDTO 分页请求信息
     * @return 分页信息
     */
    public Optional<PageDataRespDTO<DataResourceInfoDTO>> getPage(PageReqDTO pageReqDTO) {
        String url = dataResourceConfig.getDataResource().getDataCatalogUrl() + CATALOG_LIST_URL;
        log.info("Request dataCatalog page info:{} ", JsonUtils.toJSONString(pageReqDTO));
        RequestEntity<PageReqDTO> httpEntity = new RequestEntity<>(pageReqDTO, HttpMethod.POST, URI.create(url));
        ParameterizedTypeReference<BaseResp<PageDataRespDTO<DataResourceInfoDTO>>> typeReference =
                new ParameterizedTypeReference<BaseResp<PageDataRespDTO<DataResourceInfoDTO>>>() {
                };
        ResponseEntity<BaseResp<PageDataRespDTO<DataResourceInfoDTO>>> responseEntity = restTemplate.exchange(httpEntity, typeReference);
        if (!responseEntity.getStatusCode().is2xxSuccessful()) {
            log.error("Request dataCatalog error httpStatusCode :{}", responseEntity.getStatusCode().value());
            return Optional.empty();
        }
        BaseResp<PageDataRespDTO<DataResourceInfoDTO>> baseResp = responseEntity.getBody();
        if (baseResp == null) {
            return Optional.empty();
        }
        if (baseResp.isFail()) {
            log.error("Request dataCatalog error code:{} message: {}", baseResp.getCode(), baseResp.getMessage());
        }

        return Optional.ofNullable(baseResp.getResultData());
    }

    /**
     * 获取资源信息
     *
     * @param paramMap 请求参数 请求参数
     * @param type     类型 类型
     * @param <T>      泛型
     * @return 资源信息
     */
    @Override
    public <T> T getDataResource(Map<String, Object> paramMap, Type type) {
        String url = dataResourceConfig.getDataResource().getDataCatalogUrl() + RESOURCE_TYPE;

        String response = HttpUtil.postJSON(url, JSON.toJSONString(paramMap), null, null);
        log.info("getDataResource response:{}", response);
        BaseResp<String> baseResp = JSON.parseObject(response, new TypeToken<BaseResp<String>>() {
        }.getType());
        if (baseResp.isSuccess()) {
            return JSON.parseObject(baseResp.getResultData(), type);
        }
        log.error("调用数据失败：getDataResource type error message:{} ", baseResp.getMessage());
        return null;
    }

}
