package com.dragoninfo.dcuc.app.service.sub.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragoninfo.dcuc.app.entity.ServiceResource;
import com.dragoninfo.dcuc.app.entity.sub.AppDataItemInfo;
import com.dragoninfo.dcuc.app.entity.sub.AppDataResourceInfo;
import com.dragoninfo.dcuc.app.mapper.sub.AppDataItemInfoMapper;
import com.dragoninfo.dcuc.app.service.sub.IAppDataItemInfoService;
import com.dragoninfo.dcuc.app.service.sub.IAppDataResourceInfoService;
import com.dragoninfo.dcuc.app.vo.AppDataItemPageVo;
import com.dragoninfo.dcuc.common.utils.DcucBeanUtil;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2022/10/2
 */
@Service
public class AppDataItemInfoServiceImpl implements IAppDataItemInfoService {

    @Autowired
    private AppDataItemInfoMapper appDataItemInfoMapper;

    @Autowired
    private IAppDataResourceInfoService appDataResourceInfoService;

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void saveAll(List<AppDataItemInfo> appDataItemInfoList) {
        for (AppDataItemInfo appDataItemInfo : appDataItemInfoList) {
            appDataItemInfoMapper.insert(appDataItemInfo);
        }
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void deleteAll() {
        appDataItemInfoMapper.delete(Wrappers.emptyWrapper());
    }

    @Override
    public AppDataItemInfo selectByDataItemNo(String dataItemNo) {
        LambdaQueryWrapper<AppDataItemInfo> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(AppDataItemInfo::getDataItemNo, dataItemNo);
        return appDataItemInfoMapper.selectOne(lambdaQueryWrapper);
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void updateAll(List<AppDataItemInfo> updateDataItemInfoList) {
        for (AppDataItemInfo appDataItemInfo : updateDataItemInfoList) {
            appDataItemInfoMapper.updateById(appDataItemInfo);
        }
    }

    @Override
    public List<AppDataItemInfo> searchList(Searchable searchable) {
        searchable.removePageable();
        return appDataItemInfoMapper.findBySearchable(searchable);
    }

    @Override
    public List<AppDataItemInfo> selectByTableId(String tableId) {
        LambdaQueryWrapper<AppDataItemInfo> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(AppDataItemInfo::getResourceTableId, tableId);
        return appDataItemInfoMapper.selectList(lambdaQueryWrapper);
    }

    @Override
    public Long count(Date startTime, Date endTime) {
        LambdaQueryWrapper<AppDataItemInfo> query = Wrappers.lambdaQuery();
        if (null != startTime) {
            query.ge(AppDataItemInfo::getCreateTime, startTime);
        }
        if (null != endTime) {
            query.le(AppDataItemInfo::getCreateTime, endTime);
        }

        return Long.valueOf(appDataItemInfoMapper.selectCount(query));
    }

    @Override
    public Page<AppDataItemPageVo> pageSearch(SearchDTO searchDTO) {
        Searchable searchable = Searchable.toSearchable(searchDTO);
        Page<AppDataItemInfo> appDataItemInfos = appDataItemInfoMapper.pagingBySearchable(searchable);
        Page<AppDataItemPageVo> page = DcucBeanUtil.createCopyToObjectPage(appDataItemInfos, AppDataItemPageVo.class);
        // 查询数据集资源
        List<String> tableIds = page.getContent()
                .stream()
                .map(AppDataItemPageVo::getResourceTableId).distinct()
                .collect(Collectors.toList());
        List<AppDataResourceInfo> tableInfos = appDataResourceInfoService.getByIds(tableIds);
        Map<String, String> tableNameMap = tableInfos.stream()
                .collect(Collectors.toMap(AppDataResourceInfo::getId, AppDataResourceInfo::getResourceName));
        page.getContent().forEach(e -> {
            String resourceTableId = e.getResourceTableId();
            e.setAppDataResourceName(tableNameMap.get(resourceTableId));
        });

        return page;
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void deleteById(String id) {
        appDataItemInfoMapper.deleteById(id);
    }

    @Override
    public List<AppDataItemInfo> getHasClassifyColInfos() {
        LambdaQueryWrapper<AppDataItemInfo> query = Wrappers.lambdaQuery();
        query.select(AppDataItemInfo::getId, AppDataItemInfo::getColumnName, AppDataItemInfo::getResourceTableId,
                AppDataItemInfo::getColumnClassify, AppDataItemInfo::getDataSecurityLevel,
                AppDataItemInfo::getStandardDataItemName);
        query.isNotNull(AppDataItemInfo::getColumnClassify);
        List<AppDataItemInfo> appDataItemInfos = appDataItemInfoMapper.selectList(query);
        return appDataItemInfos
                .stream()
                .filter(e -> StringUtils.isNotEmpty(e.getColumnClassify()))
                .collect(Collectors.toList());
    }
}
