package com.dragoninfo.dcuc.app.service.sub.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.dragoninfo.dcuc.app.cons.CommonCons;
import com.dragoninfo.dcuc.app.enumresources.DataAuthTypeCodeEnum;
import com.dragoninfo.dcuc.app.dto.sub.FieldClaAcceptDTO;
import com.dragoninfo.dcuc.app.dto.sub.FieldClaDTO;
import com.dragoninfo.dcuc.app.entity.sub.FieldCla;
import com.dragoninfo.dcuc.app.enumresources.sub.FieldClaTypeEnum;
import com.dragoninfo.dcuc.app.mapper.sub.FieldClaMapper;
import com.dragoninfo.dcuc.app.service.sub.IFieldClaService;
import com.dragoninfo.dcuc.app.vo.DataResourceSyncTreeVo;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.collections.CollectionUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.core.search.filter.Condition;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author mazq
 * @date 2021/5/31
 */
@Service
public class FieldClaServiceImpl implements IFieldClaService {

    @Autowired
    private FieldClaMapper fieldClaMapper;

    @Override
    public ResponseStatus fieldClassifyAdd(FieldClaAcceptDTO dto) {
        boolean saveFlag = false;
        String firId;
        String path = null;
        FieldCla fir = getByCodeAndPath(dto.getFirCode(), path);
        if(null == fir) {
            saveFlag = true;
            FieldClaDTO fieldClaDTO = new FieldClaDTO(dto.getFirCode(),dto.getFirName(),
                    FieldClaTypeEnum.FIR.getLevel(), FieldClaTypeEnum.FIR.getCode());
            fieldClaDTO.setDataType(DataAuthTypeCodeEnum.AUTH_DATA_FIELD_FIR.value);
            FieldClaDTO firDTO = save(fieldClaDTO);
            firId = firDTO.getId();
        } else {
            firId = fir.getId();
        }
        path = firId;
        FieldCla sec = null;
        if(!saveFlag) {
            sec = getByCodeAndPath(dto.getSecCode(), path);
        }
        if (null != sec) {
            return ResponseStatus.fail("二级代码已存在");
        }
        FieldClaDTO fieldClaDTO = new FieldClaDTO(dto.getSecCode(),dto.getSecName(),
                FieldClaTypeEnum.SEC.getLevel(), FieldClaTypeEnum.SEC.getCode());
        fieldClaDTO.setFirId(firId);
        fieldClaDTO.setRemark(dto.getRemark());
        fieldClaDTO.setDataType(DataAuthTypeCodeEnum.AUTH_DATA_FIELD_SEC.value);
        save(fieldClaDTO);
        return ResponseStatus.success();
    }

    @Override
    public FieldClaDTO save(FieldClaDTO dto) {
        FieldCla fieldCla = new FieldCla();
        BeanUtils.copyProperties(dto,fieldCla);
        fieldCla.setCreateTime(new Date());
        fieldCla.setDeleted(BooleanEnum.FALSE.value);
        fieldClaMapper.insert(fieldCla);
        dto.setId(fieldCla.getId());
        return dto;
    }

    @Override
    public Boolean fieldClassifyUpdate(FieldClaAcceptDTO dto) {
        String firId = dto.getFirId();
        String secId = dto.getSecId();
        if(StringUtils.isNotBlank(firId)) {
            FieldClaDTO fieldClaDTO = new FieldClaDTO();
            fieldClaDTO.setId(firId);
            fieldClaDTO.setName(dto.getFirName());
            fieldClaDTO.setCode(dto.getFirCode());
            updateCodeName(fieldClaDTO);
        }
        if(StringUtils.isNotBlank(secId)) {
            FieldClaDTO fieldClaDTO = new FieldClaDTO();
            fieldClaDTO.setId(secId);
            fieldClaDTO.setName(dto.getSecName());
            fieldClaDTO.setCode(dto.getSecCode());
            fieldClaDTO.setRemark(dto.getRemark());
            updateCodeName(fieldClaDTO);
        }
        return true;
    }

    @Override
    public FieldClaAcceptDTO detailAllLevel(String id) {
        FieldClaAcceptDTO dto = new FieldClaAcceptDTO();
        FieldCla fieldCla = fieldClaMapper.selectById(id);
        FieldCla fir;
        if(null == fieldCla) {
            return dto;
        }
        int level = Integer.parseInt(fieldCla.getLevel());
        if(CommonCons.PATH_LEVEL_SEC == level) {
            dto.setSecCode(fieldCla.getCode());
            dto.setSecId(fieldCla.getId());
            dto.setSecName(fieldCla.getName());
            dto.setRemark(fieldCla.getRemark());
            fir = fieldClaMapper.selectById(fieldCla.getFirId());
        } else {
            fir = fieldCla;
        }
        dto.setFirCode(fir.getCode());
        dto.setFirName(fir.getName());
        dto.setFirId(fir.getId());
        return dto;
    }

    @Override
    public FieldCla getByCodeAndPath(String code, String path) {
        LambdaQueryWrapper<FieldCla> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().eq(FieldCla::getCode, code);
        FieldClaTypeEnum fieldClaTypeEnum;
        if(StringUtils.isBlank(path)) {
            fieldClaTypeEnum = FieldClaTypeEnum.FIR;
        } else {
            fieldClaTypeEnum = FieldClaTypeEnum.SEC;
            queryWrapper.eq(FieldCla::getFirId, path);
        }
        queryWrapper.eq(FieldCla::getLevel, fieldClaTypeEnum.getLevel());
        FieldCla fieldCla = fieldClaMapper.selectOne(queryWrapper);
        return fieldCla;
    }

    @Override
    public Boolean deleteAllLevel(String id) {
        FieldCla fieldCla = fieldClaMapper.selectById(id);
        if(null == fieldCla) {
            return false;
        }
        delete(fieldCla.getId());
        deleteChildLevel(fieldCla);
        return true;
    }

    @Override
    public List<FieldClaDTO> getLevelList(String upId, String level) {
        LambdaQueryWrapper<FieldCla> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().eq(FieldCla::getLevel, level);
        if(StringUtils.isBlank(upId)) {
            queryWrapper.eq(FieldCla::getFirId, upId);
        }
        List<FieldCla> fieldClas = fieldClaMapper.selectList(queryWrapper);
        List<FieldClaDTO> collect = fieldClas.stream().map(item -> {
            FieldClaDTO dto = new FieldClaDTO();
            BeanUtils.copyProperties(item, dto);
            return dto;
        }).collect(Collectors.toList());
        return collect;
    }

    @Override
    public Page<FieldClaAcceptDTO> pageSearch(SearchDTO searchDTO) {
        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.addSearchFilter("level", SearchOperator.eq,FieldClaTypeEnum.SEC.getLevel());
        //一级分类名称
        Condition firName = searchable.getSearchFilter("firName", SearchOperator.like);
        if(null != firName) {
            List<FieldCla> list = getByNameLike(firName.getValue().toString(), FieldClaTypeEnum.FIR.getLevel());
            if(CollectionUtils.isEmpty(list)) {
                return new PageImpl<>(new ArrayList<>(),searchable.getPage(),0);
            }
            List<String> ids = Optional.ofNullable(list).orElse(new ArrayList<>())
                    .stream().map(FieldCla::getId).distinct().collect(Collectors.toList());
            searchable.addSearchFilter("fir_id", SearchOperator.in, ids);
            searchable.removeSearchFilter("firName", SearchOperator.like);
        }
        //二级分类名称
        Condition secName = searchable.getSearchFilter("secName", SearchOperator.like);
        if(null != secName) {
            searchable.addSearchFilter("name", SearchOperator.like, secName.getValue().toString());
            searchable.removeSearchFilter("secName", SearchOperator.like);
        }
        Page<FieldCla> page = fieldClaMapper.pagingBySearchable(searchable);
        List<FieldCla> content = page.getContent();
        List<String> firIds = content.stream().map(FieldCla::getFirId).distinct().collect(Collectors.toList());
        List<FieldCla> firs = getByIds(firIds);
        firs.addAll(content);
        List<FieldClaAcceptDTO> dtos = convertToTableList(firs);
        return new PageImpl<>(dtos, page.getPageable(), page.getTotalElements());
    }

    @Override
    public List<FieldCla> getByNameLike(String name, String level) {
        LambdaQueryWrapper<FieldCla> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().like(FieldCla::getName, name);
        if(StringUtils.isNotBlank(level)) {
            queryWrapper.eq(FieldCla::getLevel, level);
        }
        return fieldClaMapper.selectList(queryWrapper);
    }

    private List<FieldClaAcceptDTO> convertToTableList(List<FieldCla> fieldClas) {
        if(CollectionUtils.isEmpty(fieldClas)) {
            return new ArrayList<>();
        }
        Map<String, List<FieldCla>> levelMap = Optional.ofNullable(fieldClas)
                .orElse(new ArrayList<>())
                .stream().collect(Collectors.groupingBy(FieldCla::getLevel));
        List<FieldCla> secs = levelMap.get(FieldClaTypeEnum.SEC.getLevel());
        List<FieldCla> firs = levelMap.get(FieldClaTypeEnum.FIR.getLevel());
        Map<String, FieldCla> firIdsMap = Optional.ofNullable(firs).orElse(new ArrayList<>())
                .stream()
                .collect(Collectors.toMap(FieldCla::getId, item -> item));
        List<FieldClaAcceptDTO> dtos = new ArrayList<>();
        for (FieldCla sec : secs) {
            FieldClaAcceptDTO dto = new FieldClaAcceptDTO();
            dto.setSecId(sec.getId());
            dto.setSecCode(sec.getCode());
            dto.setSecName(sec.getName());
            dto.setRemark(sec.getRemark());
            FieldCla fir = firIdsMap.get(sec.getFirId());
            if(null != fir) {
                dto.setFirId(fir.getId());
                dto.setFirCode(fir.getCode());
                dto.setFirName(fir.getName());
            }
            dtos.add(dto);
        }
        return dtos;
    }

    @Override
    public List<FieldCla> getByIds(List<String> ids) {
        if(CollectionUtils.isEmpty(ids)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<FieldCla> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().in(FieldCla::getId,ids);
        return fieldClaMapper.selectList(queryWrapper);
    }

    @Override
    public List<FieldCla> findList(Searchable searchable) {
        return fieldClaMapper.findBySearchable(searchable);
    }

    @Override
    public void sync(List<DataResourceSyncTreeVo> nodes, String path) {
        if(CollectionUtils.isEmpty(nodes)) {
            return;
        }
        for (DataResourceSyncTreeVo node : nodes) {
            FieldCla fieldCla = getByCodeAndPath(node.getCode(), path);
            String localPath = path;
            if(null == fieldCla) {
                saveNodeAndChild(node, localPath, new Date());
            } else {
                updateNode(node, fieldCla);
                localPath = localPath == null? fieldCla.getId():path + CommonCons.PATH_SEP + fieldCla.getId();
                sync(node.getChild(), localPath);
            }
        }
    }

    @Override
    public List<FieldClaAcceptDTO> getAll() {
        LambdaQueryWrapper<FieldCla> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().eq(FieldCla::getLevel, FieldClaTypeEnum.SEC.getLevel());
        List<FieldCla> list = fieldClaMapper.selectList(queryWrapper);
        List<String> firIds = list.stream().map(FieldCla::getFirId).distinct().collect(Collectors.toList());
        List<FieldCla> firs = getByIds(firIds);
        firs.addAll(list);
        return convertToTableList(firs);
    }

    @Override
    public List<FieldCla> getAllFieldCla() {
        return fieldClaMapper.selectList(null);
    }

    private void updateNode(DataResourceSyncTreeVo node, FieldCla fieldCla) {
        LambdaUpdateWrapper<FieldCla> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(FieldCla::getName, node.getLabel())
                .set(FieldCla::getUpdateTime, new Date())
                .eq(FieldCla::getId, fieldCla.getId());
        fieldClaMapper.update(null, updateWrapper);
    }

    /**
     * 保存树节点所有子节点
     * @param node 需要保存的顶级树节点
     * @param path 上级所有id组成的路径 -分隔
     * @param date
     */
    private void saveNodeAndChild(DataResourceSyncTreeVo node, String path, Date date) {
        FieldCla fieldCla = new FieldCla();
        fieldCla.setName(node.getLabel());
        fieldCla.setCode(node.getCode());
        fieldCla.setDeleted(BooleanEnum.FALSE.value);
        fieldCla.setCreateTime(date);
        fieldCla.setDataType(node.getDataType());
        int length = 1;
        if(StringUtils.isNotBlank(path)) {
            String[] idArr = path.split(CommonCons.PATH_SEP);
            length = idArr.length + 1;
            if(length >= CommonCons.PATH_LEVEL_FIR) {
                fieldCla.setFirId(idArr[0]);
            }
        }
        FieldClaTypeEnum typeEnum = FieldClaTypeEnum.getByLevel(length);
        if(null != typeEnum) {
            fieldCla.setLevel(typeEnum.getLevel());
            fieldCla.setType(typeEnum.getCode());
        }
        fieldClaMapper.insert(fieldCla);
        path = path == null? fieldCla.getId():path + CommonCons.PATH_SEP + fieldCla.getId();
        List<DataResourceSyncTreeVo> child = node.getChild();
        if(CollectionUtils.isNotEmpty(child)) {
            for (DataResourceSyncTreeVo childNode : child) {
                String localPath = path;
                saveNodeAndChild(childNode, localPath, date);
            }
        }
    }

    private void deleteChildLevel(FieldCla fieldCla) {
        String level = fieldCla.getLevel();
        if(FieldClaTypeEnum.SEC.getLevel().equals(level)) {
            return;
        }
        LambdaUpdateWrapper<FieldCla> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(FieldCla::getDeleteTime, new Date())
                .set(FieldCla::getDeleted, BooleanEnum.TRUE.value)
                .set(FieldCla::getDeleteUser, UserContextUtils.getCurrentUser().getId())
                .eq(FieldCla::getFirId, fieldCla.getId());
        fieldClaMapper.update(null, updateWrapper);
    }

    @Override
    public void delete(String id) {
        fieldClaMapper.deleteById(id);
    }

    @Override
    public FieldCla getById(String id) {
        return fieldClaMapper.selectById(id);
    }

    private void updateCodeName(FieldClaDTO fieldClaDTO) {
        LambdaUpdateWrapper<FieldCla> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(FieldCla::getCode, fieldClaDTO.getCode())
                .set(FieldCla::getName, fieldClaDTO.getName());
        if(StringUtils.isNotBlank(fieldClaDTO.getRemark())) {
            updateWrapper.set(FieldCla::getRemark, fieldClaDTO.getRemark());
        }
        updateWrapper.set(FieldCla::getUpdateTime, new Date());
        updateWrapper.eq(FieldCla::getId,fieldClaDTO.getId());
        fieldClaMapper.update(null, updateWrapper);
    }
}
